import React, { Component } from 'react';
import logo from './logo.svg';
import {
  Route, BrowserRouter as Router, NavLink
} from 'react-router-dom';
import { Button } from 'react-bootstrap';
import mqtt from 'mqtt';
import './App.css'
import Sensor from './Sensor'

import Home from './Home'

var client = false;

// On déclare notre tableau de capteurs
let sensorInfo = [];

const Page = ({sensor}) => {

  // On créé une array spécifique à chaque sensor (affiche toutes les données attendues pour l'historique)
  let arr = [];

  // On crée 2 variables qui permettront d'afficher le résultat sous une meilleure forme.
  var sensor_values_read = "";
  var type_sensor_value = "";

  switch(sensor.type) {
    case "TEMPERATURE": sensor_values_read = "Température actuelle : " + sensor.lastValue+"°C"; break;
    case "ON_OFF": sensor_values_read = "Etat de mes yeux : " + sensor.lastValue; break;
    case "PERCENT": sensor_values_read = "Pourcentage actuel : " + sensor.lastValue+"%"; break;
    default: sensor_values_read = "Capteur : " + sensor.name + " ; Valeur courante : " + sensor.lastValue; break;
  }

  switch(sensor.type) {
    case "TEMPERATURE": type_sensor_value = "Température"; break;
    case "ON_OFF": type_sensor_value = "On_off"; break;
    case "PERCENT": type_sensor_value = "Pourcentage %"; break;
    default: type_sensor_value = "Valeur courante"; break;
  }

  // Check si on a plusieurs valeurs à afficher dans l'historique
  if(typeof sensor.value === 'object' && sensor.value.length !== undefined) {

    /* Transformer les données dans une seul array pour chaque sensor */
    for(var i = 0; i < sensor.value.length; i++) {
        arr.push([{'labels': sensor.labels[i], 'value': sensor.value[i]}]);
    }


    var idKey = 1;

    // On affiche l'historique en Real-Time
    var historique = arr.map(function(response) {

        return (
          <tr key={idKey=idKey+1}>
          <td key={response[0].value}>
            {response[0].value}
          </td>
          <td key={response[0].labels}>
            {response[0].labels.toLocaleString('fr-FR', { timeZone: 'Europe/Paris' })}
          </td>
          </tr>
        );
    });
  }

  return (
      <div><h1>{sensor_values_read} (<i>Date de dernière mise à jour : {sensor.lastDate}</i>)</h1>

      <br />

      <table border="1">
        <thead>
        <tr>
          <th colSpan={2}><h1>Historique</h1></th>
        </tr>
        <tr>
          <th>{type_sensor_value}</th>
          <th>Dernière maj</th>
        </tr>
        </thead>
        <tbody>
          {historique || <tr><td colSpan={2}>Aucun historique disponible pour ce capteur</td></tr>}
        </tbody>
      </table></div>
  );
}

class App extends Component {

  constructor(props) {
    super(props);
    this.state = { value: 'ws://127.0.0.1:8080', error: null, messages: [], capteurs: [], sensor: [] };

    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  // Ajout de message (Payload) : Utilisé en mode DEBUG
  addMessage(message) {
      var updated = this.state.messages;
      updated.push(message);
      this.setState({messages: updated});
  }

  // Ajout de capteur dans l'état Capteurs
  addCapteur(sensor) {
      var sensorUpdated = this.state.capteurs;
      sensorUpdated.push(sensor);
      this.setState({capteurs: sensorUpdated});
  }

  // Ajout des données dans l'état Sensor
  addDataSensor(sensor) {
      var sensorUpdated = this.state.sensor;
      sensorUpdated.push(sensor);
      this.setState({sensor: sensorUpdated});
  }

  // Permet de récupèrer la liste des capteurs dispo en Real-Time
  getSensor() {
    return this.state.sensor;
  }

  // Permet de réinitialiser tous les états à leurs états initiaux
  getInitialState() {
      return { value: 'ws://', error: null, messages: [], capteurs: [], sensor: [] };
  }

  // On affiche la fonction getInitialState() pour reset les états
  reset() {
    this.setState(this.getInitialState());
  }

  // Permet de mettre à jour les informations des capteurs
  updateDataSensor(id, obj) {
    var newsensor = this.state.sensor;
    var data = {};
    data.data = obj._data;
    data.err = obj._err;
    data.name = obj._name;
    data.value = obj._value;
    data.id = obj._id;
    data.type = obj._type;
    newsensor = new Sensor(data);
    this.setState({ newsensor });
  }

  // Permet de changer l'état selon la valeur de l'input
  handleChange(event) {
    this.setState({value: event.target.value});
  }

  // Au moment de la validation du formulaire
  handleSubmit(event) {

    event.preventDefault();

    // On récupère la chaine de caractère renseigné par le client
    var debutWS = this.state.value.substring(0,5);
    var debutMQTT = this.state.value.substring(0,7);

    // On initialize self en tant que this
    var self = this;

    // Si on a déjà un client démarré on l'arrête
    if(client) { client.end(); }

    // On vérifie le début de la chaine de caractère si c'est un protocol mqtt ou ws
    if(debutWS !== 'ws://' && debutMQTT !== 'mqtt://') {
      var newValue = "ws://"+this.state.value;
      self.setState({value: newValue});
    }

    // On se connecte au client avec l'adresse pré-rempli
    client = mqtt.connect(this.state.value);

    // Une fois que l'on est connecté, on souscrit à tous les topics dispo
    client.on('connect', function() {
      client.subscribe("#");
    });

    // On reset nos états à l'état initial
    self.reset();

    // En cas d'erreur du socket on met fin à la connexion du client
    client.stream.socket.onerror = function() {
      self.setState({error: '[Erreur 500] Erreur de connexion au serveur'});
      client && client.end();
    };

    // Préfixe du Topic
    const prefixTopic = "value/";

    // On assignera pour chaque capteur un id unique et un
    var id = 1;

    // Liste des capteurs disponibles
    var listCapteurs = [];

    // Permet de vérifier le contenu d'une array si on a dispose déjà de l'element passé en paramètre
    Array.prototype.contains = function(obj) {
      var i = this.length;
      while (i--) {
          if (this[i] === obj) {
              return true;
          }
      }
      return false;
    }

    client.on('message', function(topic, payload, packet) {

        // On parse le résultat de l'objet afin de pouvoir le lire intégralement en tant qu'objet JSON
        const obj = JSON.parse(payload.toString());

        var name = topic.replace(prefixTopic,"");

        obj.id = id;
        obj.name = name;

        if(obj.type !== "ON_OFF") {
          obj.value = parseFloat(obj.value);

          if(obj.type === "PERCENT") {
            obj.value = (Math.round(obj.value * 10) / 10)*100;
          }
          else {
            obj.value = Math.round(obj.value * 10) / 10;
          }
        }

        // Variable permettant de déterminer la date la plus récente
        var firstDate = "";

        // On vérifie si le capteur reçu du message existe déjà dans notre liste de capteurs
        if(!listCapteurs.contains(name)) {

              var data = {};
              data.values = obj.value;
              data.labels = new Date();
              obj.data = data;

              // On crée notre nouveau capteur
              sensorInfo = new Sensor(obj);

              // On l'ajoute dans notre liste
              listCapteurs.push(name);

              // On ajoute nos capteurs
              self.addCapteur({

                  name: sensorInfo.name,
                  url: "/"+sensorInfo.name,
                  sensor: sensorInfo
              });

              // on ajoute dans notre liste chaque info de capteurs
              self.addDataSensor({
                  sensor: sensorInfo
              });

              id++;
        }
        else {

              // On parcours la liste de nos différents capteurs et on ajoute les nouvelles informations associées aux messages reçus
              for(var k = 0; k < self.getSensor().length; k++) {

                if(self.getSensor()[k].sensor.name === name) {

                  var date = new Date();

                  if(typeof self.getSensor()[k].sensor._data.values !== "array" && self.getSensor()[k].sensor._data.labels instanceof Date) {
                    if(self.getSensor()[k].sensor.type !== 'ON_OFF') {
                      var firstValue = self.getSensor()[k].sensor._data.values;
                      self.getSensor()[k].sensor._data.values = [];
                      self.getSensor()[k].sensor._data.values.push(firstValue);
                      self.getSensor()[k].sensor._data.values.push(obj.value);
                      firstDate = self.getSensor()[k].sensor._data.labels;
                      self.getSensor()[k].sensor._data.labels = [];
                      self.getSensor()[k].sensor._data.labels.push(firstDate);
                      self.getSensor()[k].sensor._data.labels.push(date);
                    }
                    else {
                      self.getSensor()[k].sensor._data.values = obj.value;
                      firstDate = self.getSensor()[k].sensor._data.labels;
                      self.getSensor()[k].sensor._data.labels = [];
                      self.getSensor()[k].sensor._data.labels.push(firstDate);
                      self.getSensor()[k].sensor._data.labels.push(date);
                    }
                  }
                  else {
                    if(typeof self.getSensor()[k].sensor._data.values === "string" && self.getSensor()[k].sensor.type === 'ON_OFF') {

                      // A chaque changement des yeux
                      self.getSensor()[k].sensor._data.values = obj.value;
                      self.getSensor()[k].sensor._data.labels.push(date);

                    }
                    else
                    {
                      if(self.getSensor()[k].sensor._data.values.length === 10 && self.getSensor()[k].sensor._data.labels.length === 10) {

                        for(var m = 0; m < self.getSensor()[k].sensor._data.values.length; m++) {
                            if(m !== self.getSensor()[k].sensor._data.values.length) {
                              self.getSensor()[k].sensor._data.values[m] = self.getSensor()[k].sensor._data.values[m+1];
                              self.getSensor()[k].sensor._data.labels[m] = self.getSensor()[k].sensor._data.labels[m+1];
                            }
                        }

                        self.getSensor()[k].sensor._data.values.splice(-1, 1);
                        self.getSensor()[k].sensor._data.labels.splice(-1, 1);
                        self.getSensor()[k].sensor._data.values.push(obj.value);
                        self.getSensor()[k].sensor._data.labels.push(date);
                      }
                      else {

                        self.getSensor()[k].sensor._data.values.push(obj.value);
                        self.getSensor()[k].sensor._data.labels.push(date);
                      }

                    }
                  }

                  // On met à jour à chaque fois qu'on obtient un capteur que l'on connait
                  self.updateDataSensor(k, self.getSensor()[k].sensor);

               }
              }
          }
    });
}

  render() {

    // Mon Historique de messages */
    var historique = this.state.capteurs.map(function(message) {
        return (
            <div key={message.sensor.id} className="message">{message.sensor.name}</div>
        );
    });

    /* Création de link capteurs dynamique */
    var linkCapteurs = this.state.capteurs.map(function(capteurs) {

        return (
          <li key={capteurs.name} className='OK'><NavLink to={capteurs.url} style={{display: 'block', height: '100%'}} activeClassName="active"><Button>{capteurs.name}</Button></NavLink></li>
        );
    });

    /* Création de route de capteurs dynamique */
    var routeCapteurs = this.state.capteurs.map(function(capteurs) {
        return (
          <Route key={capteurs.name} path={capteurs.url} params={capteurs.sensor} render={(props) => (
            <Page sensor={capteurs.sensor} />
          )}/>
        );
    });

    return (
      <div className="App">

            <header className="App-header"><img src={logo} className="App-logo" alt="logo" /><h1 className="App-title">URL du Brocker : <form onSubmit={this.handleSubmit}><input type="text" value={this.state.value} pattern="(ws|mqtt):\/\/(.*)" onChange={this.handleChange} required/><input type="submit" value="Tester la connexion"/></form><div className="error-message">{this.state.error || ""}</div></h1></header>

            <Router>
            <div className="wrapper">

              <nav><ul role="nav">
                  <li><NavLink to="/home" style={{display: 'block', height: '100%'}}><Button>Home</Button></NavLink></li>
                  <hr />
                  <div id="dynamiqueNav">
                    <h3>Navigation</h3>
                    <ul>{linkCapteurs}</ul>
                    <hr />
                  </div>
              </ul></nav>
              <article>
                  <div id="messageList" className="table-block footer-push">
                    <Route path="/home" component={Home}>
                    </Route>
                    {routeCapteurs}
                    <div id="content">
                    </div>
                  </div>
              </article>
            </div>
            </Router>

            <footer>Projet réalisé par Julien SAUTEREAU - Master 2 iWOCS</footer>
            </div>
          );
        }
}

export default App;
